import http
import re

from http import HTTPStatus

import aiohttp
import pytest
import responses

from quickbuild import (
    AsyncQBClient,
    ContentType,
    QBClient,
    QBError,
    QBServerError,
)
from quickbuild.exceptions import QBForbiddenError

GET_VERSION_DATA = '6.0.9'


USERS_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.model.User>
    <id>1</id>
    <favoriteDashboardIds>
      <long>1</long>
    </favoriteDashboardIds>
    <name>admin</name>
    <lastLogin>2021-06-10T14:18:21.292Z</lastLogin>
    <password secret="hash">0DPiKuNIrrVmD8IUCuw1hQxNqZc=</password>
    <pluginSettingDOMs/>
  </com.pmease.quickbuild.model.User>
  <com.pmease.quickbuild.model.User>
    <id>2</id>
    <favoriteDashboardIds/>
    <name>pbelskiy</name>
    <lastLogin>2021-06-07T14:24:49.066Z</lastLogin>
    <password secret="hash">je+glP0P8AYzGoaLZE16xTSJKC8=</password>
    <pluginSettingDOMs/>
  </com.pmease.quickbuild.model.User>
</list>
"""

USERS_JSON = """[ {
  "@class" : "com.pmease.quickbuild.model.User",
  "id" : 1,
  "favoriteDashboardIds" : [ 1 ],
  "name" : "admin",
  "lastLogin" : "2021-06-10T14:18:21.292+00:00",
  "password" : {
    "value" : "0DPiKuNIrrVmD8IUCuw1hQxNqZc=",
    "secret" : "hash"
  },
  "pluginSettingDOMs" : { }
}, {
  "@class" : "com.pmease.quickbuild.model.User",
  "id" : 2,
  "favoriteDashboardIds" : [ ],
  "name" : "pbelskiy",
  "lastLogin" : "2021-06-07T14:24:49.066+00:00",
  "password" : {
    "value" : "je+glP0P8AYzGoaLZE16xTSJKC8=",
    "secret" : "hash"
  },
  "pluginSettingDOMs" : { }
} ]
"""


@responses.activate
def test_sync_client():
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/version'),
        body=GET_VERSION_DATA,
    )

    try:
        client = QBClient(
            'http://server',
            'login',
            'password',
            timeout=10,
        )

        version = client.system.get_version()
        assert version.major == 6
        assert version.minor == 0
        assert version.patch == 9
    finally:
        client.close()


@responses.activate
def test_sync_client_retry():
    # responses library does`t support Retry mock
    # https://github.com/getsentry/responses/issues/135
    # so, just cover code of retry constructor
    statuses = [HTTPStatus.BAD_REQUEST, HTTPStatus.INTERNAL_SERVER_ERROR]

    client = QBClient(
        'http://server',
        'login',
        'password',
        retry=dict(
            total=10,
            factor=1,
            statuses=statuses,
        )
    )

    assert client.session.adapters['http://'].max_retries.status_forcelist == statuses


@pytest.mark.asyncio
async def test_async_client(aiohttp_mock):
    try:
        client = AsyncQBClient(
            'http://server',
            'login',
            'password',
            timeout=10,
        )

        aiohttp_mock.get(
            'http://server/rest/version',
            content_type='text/plain',
            body=GET_VERSION_DATA,
            status=HTTPStatus.OK,
        )

        version = await client.system.get_version()
        assert version.major == 6
        assert version.minor == 0
        assert version.patch == 9
    finally:
        await client.close()


@pytest.mark.asyncio
async def test_async_client_retry(aiohttp_mock):
    client = AsyncQBClient(
        'http://server',
        'login',
        'password',
        retry=dict(
            total=10,
            statuses=[HTTPStatus.INTERNAL_SERVER_ERROR],
        )
    )

    aiohttp_mock.get(
        'http://server/rest/version',
        content_type='text/plain',
        body='Server error',
        status=HTTPStatus.INTERNAL_SERVER_ERROR,
    )

    aiohttp_mock.get(
        'http://server/rest/version',
        content_type='text/plain',
        body=GET_VERSION_DATA,
        status=HTTPStatus.OK,
    )

    version = await client.system.get_version()
    assert version.major == 6
    assert version.minor == 0
    assert version.patch == 9
    await client.close()


@pytest.mark.asyncio
async def test_async_client_retry_exception(aiohttp_mock):
    client = AsyncQBClient(
        'http://server',
        'login',
        'password',
        retry=dict(
            total=2,
            statuses=[HTTPStatus.INTERNAL_SERVER_ERROR],
        )
    )

    aiohttp_mock.get(
        'http://server/rest/version',
        exception=aiohttp.ClientError()
    )

    aiohttp_mock.get(
        'http://server/rest/version',
        exception=aiohttp.ClientError()
    )

    with pytest.raises(QBError):
        await client.system.get_version()

    await client.close()


def test_retry_argument_validation():
    with pytest.raises(QBError):
        QBClient('http://server', retry=dict(total=1, strange_argument=1))

    with pytest.raises(QBError):
        AsyncQBClient('http://server', 'login', 'password', retry=dict(total=0))


@responses.activate
def test_content_type_parse_get():
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/users'),
        content_type='application/xml',
        body=USERS_XML,
    )

    client = QBClient('http://server', content_type=ContentType.PARSE)

    users = client.users.get()
    assert isinstance(users, list)


@responses.activate
def test_content_type_xml_get():
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/users'),
        content_type='application/xml',
        body=USERS_XML,
    )

    client = QBClient('http://server', content_type=ContentType.XML)

    users = client.users.get()
    assert isinstance(users, str)


@responses.activate
def test_content_type_json_get():
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/users'),
        content_type='application/json',
        body=USERS_JSON,
    )

    client = QBClient('http://server', content_type=ContentType.JSON)

    users = client.users.get()
    assert isinstance(users, list)


@responses.activate
def test_content_type_headers_sync_json_json():

    def callback(request):
        assert request.headers['Accept'] == 'application/json'
        return (HTTPStatus.OK, request.headers, request.body)

    responses.add_callback(
        responses.GET,
        re.compile(r'.*/rest/memberships/\d+'),
        callback=callback
    )

    client = QBClient('http://server', content_type=ContentType.JSON)
    client.memberships.get_info(1)


@responses.activate
def test_content_type_headers_sync_xml_json():

    def callback(request):
        assert request.headers['Accept'] == 'application/json'
        return (HTTPStatus.OK, request.headers, request.body)

    responses.add_callback(
        responses.GET,
        re.compile(r'.*/rest/memberships/\d+'),
        callback=callback
    )

    client = QBClient('http://server', content_type=ContentType.XML)
    client.memberships.get_info(1, content_type=ContentType.JSON)


@responses.activate
def test_content_type_headers_sync_json_xml():

    def callback(request):
        assert request.headers['Accept'] == '*/*'
        return (HTTPStatus.OK, request.headers, request.body)

    responses.add_callback(
        responses.GET,
        re.compile(r'.*/rest/memberships/\d+'),
        callback=callback
    )

    client = QBClient('http://server', content_type=ContentType.JSON)
    client.memberships.get_info(1, content_type=ContentType.XML)


@responses.activate
def test_exception_server_error(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/version'),
        body=GET_VERSION_DATA,
        status=HTTPStatus.INTERNAL_SERVER_ERROR
    )

    with pytest.raises(QBServerError):
        client.system.get_version()


@responses.activate
def test_exception_forbidden_error(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/version'),
        body=GET_VERSION_DATA,
        status=HTTPStatus.FORBIDDEN
    )

    with pytest.raises(QBForbiddenError):
        client.system.get_version()


@responses.activate
def test_exception_common_error(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/version'),
        body=GET_VERSION_DATA,
        status=HTTPStatus.METHOD_NOT_ALLOWED
    )
    with pytest.raises(QBError):
        client.system.get_version()



from functools import partial
from typing import List, Optional, Union

from quickbuild.endpoints.shares import UserShares
from quickbuild.helpers import ContentType, response2py


class Users:

    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

        self.shares = UserShares(quickbuild)

    def get(self) -> List[dict]:
        """
        Get all users in the system.

        Returns:
            List[dict]: list of users.
        """
        return self.quickbuild._request(
            'GET',
            'users',
            callback=response2py
        )

    def get_info(self,
                 user_id: int,
                 *,
                 content_type: Optional[ContentType] = None
                 ) -> Union[dict, str]:
        """
        Get information about specified user.

        Args:
            user_id (int):
                User identifier.

            content_type (Optional[ContentType]):
                Select needed content type if not set, default value of client
                instance is used.

        Returns:
            dict: information about user.
        """
        return self.quickbuild._request(
            'GET',
            'users/{}'.format(user_id),
            callback=partial(response2py, content_type=content_type),
            content_type=content_type,
        )

    def get_display_name(self, user_id: int) -> str:
        """
        Get display name for specified user.

        Args:
            user_id (int): user identifier.

        Returns:
            str: user name.
        """
        return self.quickbuild._request(
            'GET',
            'users/{}/display_name'.format(user_id)
        )

    def get_id_by_name(self, name: str) -> int:
        """
        Get user id by name.

        Args:
            name (str): user name.

        Returns:
            int: user identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self.quickbuild.identifiers.get_user_id_by_name(name)

    def update(self, configuration: str) -> int:
        """
        Update user using XML configuration.

        Normally you do not need to create the xml from scratch: you may retrieve
        XML representation of the user using ``get_info()`` method, modify certain
        parts of the XML and post back to above url.

        Args:
            configuration (str): XML document.

        Returns:
            int: user id being updated.
        """
        return self.quickbuild._request(
            'POST',
            'users',
            callback=response2py,
            data=configuration,
        )

    def create(self, configuration: str) -> int:
        """
        Create a new user using XML/JSON configuration.

        Normally you do not need to create the XML from scratch: you may
        retrieve XML representation of a templating user using ``get_info()``,
        remove the id element, modify certain parts and use it.

        Args:
            configuration (str): XML/JSON document.

        Returns:
            int: id of the newly created user.

        Raises:
            QBError: configuration validation error.
        """
        self.quickbuild._validate_for_id(configuration)
        return self.update(configuration)

    def delete(self, user_id: int) -> None:
        """
        Delete user by user id.

        Args:
            user_id (int): user identifier.

        Returns:
            None
        """
        return self.quickbuild._request(
            'DELETE',
            'users/{}'.format(user_id),
            callback=response2py,
        )



import re

import pytest
import responses

from quickbuild import QBError

USERS_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.model.User>
    <id>1</id>
    <favoriteDashboardIds>
      <long>1</long>
    </favoriteDashboardIds>
    <name>admin</name>
    <password secret="hash">0DPiKuNIrrVmD8IUCuw1hQxNqZc=</password>
    <pluginSettingDOMs/>
  </com.pmease.quickbuild.model.User>
</list>
"""

USER_INFO_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<com.pmease.quickbuild.model.User>
  <id>1</id>
  <favoriteDashboardIds>
    <long>1</long>
  </favoriteDashboardIds>
  <name>admin</name>
  <password secret="hash">0DPiKuNIrrVmD8IUCuw1hQxNqZc=</password>
  <pluginSettingDOMs/>
</com.pmease.quickbuild.model.User>
"""


@responses.activate
def test_get(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/users'),
        content_type='application/xml',
        body=USERS_XML,
    )

    response = client.users.get()
    assert len(response) == 1
    assert response[0]['id'] == 1


@responses.activate
def test_get_info(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/users/1'),
        content_type='application/xml',
        body=USER_INFO_XML,
    )

    response = client.users.get_info(1)
    assert response['id'] == 1
    assert response['name'] == 'admin'


@responses.activate
def test_get_display_name(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/users/1/display_name'),
        content_type='application/xml',
        body='admin',
    )

    response = client.users.get_display_name(1)
    assert response == 'admin'


@responses.activate
def test_update(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/users'),
        body='1',
    )

    response = client.users.update(USER_INFO_XML)
    assert response == 1


@responses.activate
def test_create(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/users'),
        body='1',
    )

    xml_with_id = USER_INFO_XML
    xml_without_id = xml_with_id.replace('<id>1</id>', '')

    with pytest.raises(QBError):
        client.users.create(xml_with_id)

    response = client.users.create(xml_without_id)
    assert response == 1


@responses.activate
def test_delete(client):
    responses.add(
        responses.DELETE,
        re.compile(r'.*/rest/users')
    )

    client.users.delete(1)



from quickbuild.adapters.aio import AsyncQBClient as AsyncQBClient
from quickbuild.adapters.sync import QBClient as QBClient
from quickbuild.exceptions import QBError as QBError, QBForbiddenError as QBForbiddenError, QBNotFoundError as QBNotFoundError, QBProcessingError as QBProcessingError, QBServerError as QBServerError
from quickbuild.helpers import ContentType as ContentType



from typing import Any

from quickbuild.helpers import response2py


class Identifiers:
    """
    Most of QuickBuild RESTful API relies on identifier of the object,
    which can be retrieved with the id service explained here.

    https://wiki.pmease.com/display/QB10/Retrieve+Object+Identifier
    """
    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def _get(self, params: dict) -> Any:
        return self.quickbuild._request(
            'GET',
            'ids',
            callback=response2py,
            params=params,
        )

    def get_resource_id_by_name(self, name: str) -> int:
        """
        Get resource id by name.

        .. note::
            This feature is available since 6.1.35

        Args:
            name (str): resource path.

        Returns:
            int: resource identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(resource_name=name))

    def get_configuration_id_by_path(self, path: str) -> int:
        """
        Get configuration id by path.

        Args:
            path (str): configuration path.

        Returns:
            int: configuration identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(configuration_path=path))

    def get_user_id_by_name(self, name: str) -> int:
        """
        Get user id by name.

        Args:
            name (str): user name.

        Returns:
            int: user identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(user_name=name))

    def get_group_id_by_name(self, name: str) -> int:
        """
        Get group id by name.

        Args:
            name (str): group name.

        Returns:
            int: group identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(group_name=name))

    def get_queue_id_by_name(self, name: str) -> int:
        """
        Get queue id by name.

        Args:
            name (str): queue name.

        Returns:
            int: queue identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(queue_name=name))

    def get_build_id_by_request_id(self, identifier: int) -> int:
        """
        Get build id by request id.

        Args:
            identifier (int): request identifier.

        Returns:
            int: build identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(request_id=identifier))

    def get_build_id_by_build_name(self, name: str) -> int:
        """
        Get build id by build name.

        For example how to get id of latest build in configuration with id 1,
        name will be `1.latest`

        Args:
            name (str): build name.

        Returns:
            int: build identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(build_name=name))

    def get_dashboard_id_by_dashboard_fqn(self, fqn: str) -> int:
        """
        Get dashboard id by dashboard fqn (fully qualified name).

        <dashboard fqn> is of the form <user id>.<dashboard name>.

        Args:
            fqn (str): fqn (fully qualified name).

        Returns:
            int: dashboard identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self._get(dict(dashboard_fqn=fqn))



from functools import partial
from typing import List, Optional, Union

from quickbuild.helpers import ContentType, response2py


class Dashboards:

    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def get(self) -> List[dict]:
        """
        Get all dashboards in the system.

        Returns:
            List[dict]: list of all dashboards.
        """
        return self.quickbuild._request(
            'GET',
            'dashboards',
            callback=response2py,
        )

    def get_info(self,
                 dashboard_id: int,
                 *,
                 content_type: Optional[ContentType] = None
                 ) -> Union[dict, str]:
        """
        Get information about specified dashboard.

        Args:
            dashboard_id (int):
                Dashboard identifier.

            content_type (Optional[ContentType]):
                Select needed content type, if not set default value of client
                instance is used.

        Returns:
            Union[dict, str]: dashboard detailed information.
        """
        return self.quickbuild._request(
            'GET',
            'dashboards/{}'.format(dashboard_id),
            callback=partial(response2py, content_type=content_type),
            content_type=content_type,
        )

    def get_id_by_fqn(self, fqn: str) -> int:
        """
        Get dashboard id by dashboard fqn (fully qualified name).

        <dashboard fqn> is of the form <user id>.<dashboard name>.

        Args:
            fqn (str): fqn (fully qualified name).

        Returns:
            int: dashboard identifier.

        Raises:
            QBProcessingError: will be raised if resource is not found.
        """
        return self.quickbuild.identifiers.get_dashboard_id_by_dashboard_fqn(fqn)

    def update(self, configuration: str) -> int:
        """
        Update a dashboard using XML/JSON configuration.

        Normally you do not need to create the configuration from scratch, you
        may retrieve it representation of the dashboard using `get_info()`,
        modify certain parts and use it as new configuration.

        Args:
            configuration (str): XML/JSON document.

        Returns:
            int: dashboard id being updated.
        """
        return self.quickbuild._request(
            'POST',
            'dashboards',
            callback=response2py,
            data=configuration
        )

    def create(self, configuration: str) -> int:
        """
        Create a dashboard using XML/JSON configuration.

        Please note that the posted configuration should NOT contain the id
        element; otherwise, QuickBuild will treat the post as an updating to the
        dashboard with that id.

        Normally you do not need to create the configuration from scratch, you
        may retrieve it representation of the dashboard using `get_info()`,
        modify certain parts and use it as new configuration.

        Args:
            configuration (str): XML/JSON document.

        Returns:
            int: dashboard id being created.

        Raises:
            QBError: configuration validation error
        """
        self.quickbuild._validate_for_id(configuration)

        return self.quickbuild._request(
            'POST',
            'dashboards',
            callback=response2py,
            data=configuration
        )

    def delete(self, dashboard_id: int) -> None:
        """
        Delete specified dashboard.

        Args:
            dashboard_id (int): dashboard identifier.

        Returns:
            int: dashboard id being deleted.
        """
        return self.quickbuild._request(
            'DELETE',
            'dashboards/{}'.format(dashboard_id),
            callback=response2py,
        )



from functools import partial
from typing import List, Optional, Union

from quickbuild.helpers import ContentType, response2py


class Memberships:

    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def get(self) -> List[dict]:
        """
        Get all memberships in the system.

        Returns:
            List[dict]: list of all memberships.
        """
        return self.quickbuild._request(
            'GET',
            'memberships',
            callback=response2py,
        )

    def get_info(self,
                 membership_id: int,
                 *,
                 content_type: Optional[ContentType] = None
                 ) -> Union[dict, str]:
        """
        Get full membership info.

        Args:
            membership_id (int):
                Membership identifier.

            content_type (Optional[ContentType]):
                Select needed content type if not set, default value of client
                instance is used.

        Returns:
            Union[str, dict]: membership content.
        """
        return self.quickbuild._request(
            'GET',
            'memberships/{}'.format(membership_id),
            callback=partial(response2py, content_type=content_type),
            content_type=content_type,
        )

    def get_by_user(self, user_id: int) -> Union[List[dict], str]:
        """
        Get memberships of particular user.

        Args:
            user_id (int): user identifier.

        Returns:
            Union[List[dict], str]: user membership content.
        """
        return self.quickbuild._request(
            'GET',
            'memberships',
            params=dict(user_id=user_id),
            callback=response2py,
        )

    def get_by_group(self, group_id: int) -> Union[List[dict], str]:
        """
        Get memberships of particular group.

        Args:
            group_id (int): group identifier.

        Returns:
            Union[List[dict], str]: group membership content.
        """
        return self.quickbuild._request(
            'GET',
            'memberships',
            params=dict(group_id=group_id),
            callback=response2py,
        )

    def update(self, configuration: str) -> int:
        """
        Update membership using XML/JSON representation of the membership.

        Normally you do not need to create the representation from scratch, you
        may retrieve XML/JSON representation of the membership using `get_info()`
        method, modify certain parts and use it as new representation for update.

        Demo:
        -----
        1. Assume id of user robin is 2, id of group developer is 1, and id of
           group tester is 2.
        2. Get memberships of user robin using `get_by_user(2)` method.
        3. Analyze response of above command to find out the membership with
           group id.
        4. Modify response and change the group element to use value of 2.
        5. Use it as new membership membership to this `update()` method.

        Args:
            representation (str): representation of membership.

        Returns:
            int: membership id being updated.
        """
        return self.quickbuild._request(
            'POST',
            'memberships',
            callback=response2py,
            data=configuration,
        )

    def create(self, configuration: str) -> int:
        """
        Create membership using XML/JSON representation of the membership.

        Normally you do not need to create the representation from scratch, you
        may retrieve XML/JSON representation of the membership using `get_info()`
        method, modify certain parts and use it as new representation for create.

        Demo:
        -----
        How to add user robin (assume id is 2) to group tester (assume the id is 3).

        .. code-block:: xml
            <com.pmease.quickbuild.model.Membership>
              <user>2</user>
              <group>3</group>
            </com.pmease.quickbuild.model.Membership>

        Args:
            representation (str): representation of membership.

        Returns:
            int: newly created membership id.
        """
        return self.update(configuration)

    def delete(self, membership_id: int) -> None:
        """
        Delete membership.

        Demo:
        -----
        1. User robin (assume id is 2) from group tester (assume the id is 3).
        2. Get memberships of user robin with `get_by_user(2)`
        3. Analyze response of above command to find out id of the membership
           associated with group id 3, assume id of the found membership is 100.
        4. Delete the found membership id 100.

        Args:
            membership_id (str): membership identifier.
        """
        return self.quickbuild._request(
            'DELETE',
            'memberships/{}'.format(membership_id),
            callback=response2py,
        )



from typing import Dict, List, Optional, Union

from quickbuild.helpers import response2py


class Audits:

    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def get(self,
            count: int,
            *,
            username: Optional[str] = None,
            source: Optional[str] = None,
            action: Optional[str] = None,
            since: Optional[str] = None,
            until: Optional[str] = None,
            first: Optional[int] = None
            ) -> List[dict]:
        """
        Get all users in the system.

        Args:
            count (int):
                Specified number of audit entries to return. This param must be
                specified in order not to mistakenly return all audits to stress
                the system.

            username (Optional[str]):
                Name of the user to audit. If not specified, audit log of all
                users will be searched.

            source (Optional[str]):
                Specify source of audit to match. The character * can be used in
                the source string to do wildcard match. If not specified, audits
                from all sources will be matched.

            action (Optional[str]):
                Action of the audit to match. If left empty, any action will be
                matched.

            since (Optional[str]):
                In the format of yyyy-MM-dd HH:mm, for example: 2009-11-12 13:00.
                If specified, search audits generated after this date.

            until (Optional[str]):
                In the format of yyyy-MM-dd HH:mm, for example: 2009-11-12 14:00.
                If specified, search builds generated before this date.

            first (Optional[int]):
                Specified first audit entry to return. If not specified, value
                0 is assumed.

        Returns:
            List[dict]: list of audits.
        """
        params = dict(count=count)  # type: Dict[str, Union[str, int]]

        if username:
            params['username'] = username

        if source:
            params['source'] = source

        if action:
            params['action'] = action

        if since:
            params['since'] = since

        if until:
            params['until'] = until

        if first:
            params['first'] = first

        return self.quickbuild._request(
            'GET',
            'audits',
            callback=response2py,
            params=params,
        )

    def count(self) -> int:
        """
        Get count of audits.

        Returns:
            int: count of audits.
        """
        return self.quickbuild._request(
            'GET',
            'audits/count',
            callback=response2py,
        )



import datetime

from functools import partial
from typing import List, Optional, Union

from quickbuild.helpers import ContentType, response2py


class Configurations:

    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def _get(self, params: dict) -> List[dict]:
        return self.quickbuild._request(
            'GET',
            'configurations',
            callback=response2py,
            params=params,
        )

    def get(self) -> List[dict]:
        """
        Get all configurations in the system. For performance reason, only
        brief  information of the configuration will be returned here, including
        `id`, `name`, `description`, `schedule`, `runMode`, `errorMessage`,
        `parent id`. You may get the full xml representation using id if necessary.

        Returns:
            List[dict]: list of configurations.
        """
        return self._get(dict(recursive=True))

    def get_child(self, parent_id: int) -> List[dict]:
        """
        Get a list of child configurations.

        Args:
            parent_id (int): parent configuration identifier.

        Returns:
            List[dict]: list of child configurations.
        """
        return self._get(dict(parent_id=parent_id))

    def get_descendent(self, parent_id: int) -> List[dict]:
        """
        Get a list of descendent configurations.

        Args:
            parent_id (int): parent configuration identifier.

        Returns:
            List[dict]: list of descendent configurations.
        """
        return self._get(dict(recursive=True, parent_id=parent_id))

    def get_info(self,
                 configuration_id: int,
                 *,
                 content_type: Optional[ContentType] = None
                 ) -> Union[dict, str]:
        """
        Get full configuration info.

        Args:
            configuration_id (int):
                Configuration identifier.

            content_type (Optional[ContentType]):
                Select needed content type if not set, default value of client
                instance is used.

        Returns:
            Union[dict, str]: configuration content.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}'.format(configuration_id),
            callback=partial(response2py, content_type=content_type),
            content_type=content_type,
        )

    def get_path(self, configuration_id: int) -> str:
        """
        Get configuration path.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            str: configuration path.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}/path'.format(configuration_id),
        )

    def get_id_by_path(self, path: str) -> int:
        """
        Get configuration id by path.

        Args:
            path (str): configuration path.

        Returns:
            int: configuration identifier.
        """
        return self.quickbuild.identifiers.get_configuration_id_by_path(path)

    def get_name(self, configuration_id: int) -> str:
        """
        Get configuration name.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            str: configuration name.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}/name'.format(configuration_id),
        )

    def get_description(self, configuration_id: int) -> str:
        """
        Get configuration description.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            str: configuration description.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}/description'.format(configuration_id),
        )

    def get_error_message(self, configuration_id: int) -> str:
        """
        Get configuration error message.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            str: configuration error message.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}/error_message'.format(configuration_id),
        )

    def get_run_mode(self, configuration_id: int) -> str:
        """
        Get configuration run mode.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            str: configuration run mode.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}/run_mode'.format(configuration_id),
        )

    def get_schedule(self, configuration_id: int) -> dict:
        """
        Get configuration schedule.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            dict: configuration schedule.

        Raises:
            QBProcessingError: will be raised if schedule is inherited from
                               parent configuration.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}/schedule'.format(configuration_id),
            callback=response2py,
        )

    def get_average_duration(self,
                             configuration_id: int,
                             *,
                             from_date: Optional[datetime.date],
                             to_date: Optional[datetime.date]
                             ) -> int:
        """
        Get configuration average duration.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            int: milliseconds of average build duration.
        """
        params = dict()

        if from_date:
            params['from_date'] = str(from_date)

        if to_date:
            params['to_date'] = str(to_date)

        return self.quickbuild._request(
            'GET',
            'configurations/{}/average_duration'.format(configuration_id),
            callback=response2py,
            params=params,
        )

    def get_success_rate(self,
                         configuration_id: int,
                         *,
                         from_date: Optional[datetime.date],
                         to_date: Optional[datetime.date]
                         ) -> int:
        """
        Get configuration success rate.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            int: value in the range of 0~100, with 0 stands for 0%, and 100
                 stands for 100%.
        """
        params = dict()

        if from_date:
            params['from_date'] = str(from_date)

        if to_date:
            params['to_date'] = str(to_date)

        return self.quickbuild._request(
            'GET',
            'configurations/{}/success_rate'.format(configuration_id),
            callback=response2py,
            params=params,
        )

    def get_parent(self, configuration_id: int) -> int:
        """
        Get parent configuration id.

        Args:
            configuration_id (int): configuration identifier.

        Returns:
            int: id of parent configuration.

        Raises:
            QBProcessingError: the configuration is root configuration and does
                               not have parent.
        """
        return self.quickbuild._request(
            'GET',
            'configurations/{}/parent'.format(configuration_id),
            callback=response2py,
        )

    def update(self, configuration: str) -> int:
        """
        Update a configuration using XML configuration.

        Normally you do not need to create the XML from scratch: you may get
        XML representation of the configuration using `get_info()` method with
        content_type=ContentType.XML and modify certain parts of the XML.

        Args:
            configuration (str): XML document.

        Returns:
            int: configuration id being updated.
        """
        return self.quickbuild._request(
            'POST',
            'configurations',
            callback=response2py,
            data=configuration
        )

    def create(self, configuration: str) -> int:
        """
        Create a configuration using XML/JSON configuration.

        Please note that:

        - The parent element denotes id of the parent configuration. Normally
          you do not need to create the xml from scratch: you may retrieve xml
          representation of a templating configuration using various configuration
          access methods or `get_info()` with content_type=ContentType.XML, remove
          the id element, modify certain parts and use it for create() method.

        - Secret elements (Elements with attribute "secret=encrypt" in XML
          representation of an existing configuration, typically they are
          repository passwords, secret variable values, etc.) should not contain
          the "secret" attribute; otherwise QuickBuild will think that the password
          has already been encrypted. However if you creating configuration by
          copying existing one and want to remain the passwords, the "secret"
          attribute should then be preserved.

        Args:
            configuration (str): XML/JSON document.

        Returns:
            int: configuration id of newly created configuration.

        Raises:
            QBError: XML validation error
        """
        self.quickbuild._validate_for_id(configuration)
        return self.update(configuration)

    def delete(self, configuration_id: int) -> None:
        """
        Delete configuration.

        Args:
            configuration_id (int): configuration id.

        Returns:
            None
        """
        return self.quickbuild._request(
            'DELETE',
            'configurations/{}'.format(configuration_id),
            callback=response2py,
        )

    def copy(self,
             configuration_id: int,
             parent_id: int,
             name: str,
             recursive: bool
             ) -> int:
        """
        Copy configuration (available since version 4.0.72)

        Args:
            configuration_id (int):
                Configuration id to be copied.

            parent_id (int):
                Configuration id of the parent to place newly copied configuration.

            name (str):
                Name of the newly copied configuration.

            recursive (bool):
                Specify parameter recursive=true to copy specified configuration
                and all its descendant configurations recursively; otherwise,
                only the configuration itself will be copied.

        Returns:
            int: configuration id of the newly copied configuration.
        """
        params = dict(
            parent_id=parent_id,
            name=name,
            recursive=recursive,
        )

        return self.quickbuild._request(
            'GET',
            'configurations/{}/copy'.format(configuration_id),
            callback=response2py,
            params=params,
        )



import re

import pytest
import responses

from quickbuild import QBError
from quickbuild.helpers import ContentType

GROUPS_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.model.Group>
    <id>1</id>
    <name>test</name>
    <admin>false</admin>
    <restAccessAllowed>true</restAccessAllowed>
    <nodeAttributesAllowed>false</nodeAttributesAllowed>
    <systemLogAllowed>false</systemLogAllowed>
    <expandAllAllowed>false</expandAllAllowed>
    <agentAllowed>false</agentAllowed>
    <scriptAllowed>false</scriptAllowed>
    <shareDashboardAllowed>false</shareDashboardAllowed>
    <viewAlertsAllowed>false</viewAlertsAllowed>
    <forceHttps>false</forceHttps>
    <pluginSettingDOMs/>
  </com.pmease.quickbuild.model.Group>
</list>
"""

GROUP_INFO_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<com.pmease.quickbuild.model.Group>
  <id>1</id>
  <name>test</name>
  <admin>false</admin>
  <restAccessAllowed>true</restAccessAllowed>
  <nodeAttributesAllowed>false</nodeAttributesAllowed>
  <systemLogAllowed>false</systemLogAllowed>
  <expandAllAllowed>false</expandAllAllowed>
  <agentAllowed>false</agentAllowed>
  <scriptAllowed>false</scriptAllowed>
  <shareDashboardAllowed>false</shareDashboardAllowed>
  <viewAlertsAllowed>false</viewAlertsAllowed>
  <forceHttps>false</forceHttps>
  <pluginSettingDOMs/>
</com.pmease.quickbuild.model.Group>
"""


@responses.activate
def test_get(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/groups'),
        content_type='application/xml',
        body=GROUPS_XML,
    )

    response = client.groups.get()
    assert len(response) == 1
    assert response[0]['id'] == 1
    assert response[0]['admin'] is False


@responses.activate
def test_get_info_as_json(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/groups/1'),
        content_type='application/xml',
        body=GROUP_INFO_XML,
    )

    response = client.groups.get_info(1)
    assert response['id'] == 1


@responses.activate
def test_get_info_as_xml(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/groups/1'),
        content_type='application/xml',
        body=GROUP_INFO_XML,
    )

    response = client.groups.get_info(1, content_type=ContentType.XML)
    assert isinstance(response, str)


@responses.activate
def test_update(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/groups'),
        content_type='text/plain',
        body='1',
    )

    response = client.groups.update(GROUP_INFO_XML)
    assert response == 1


@responses.activate
def test_create(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/groups'),
        content_type='text/plain',
        body='1',
    )

    xml_with_id = GROUP_INFO_XML
    xml_without_id = xml_with_id.replace('<id>1</id>', '')

    with pytest.raises(QBError):
        client.groups.create(xml_with_id)

    response = client.groups.create(xml_without_id)
    assert response == 1


@responses.activate
def test_delete(client):
    responses.add(
        responses.DELETE,
        re.compile(r'.*/rest/groups/(\d+)'),
    )

    client.groups.delete(1)



import re

import pytest
import responses

from quickbuild import QBError

DASHBOARDS_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.model.Dashboard>
    <id>1</id>
    <user>1</user>
    <name>Default</name>
    <description>System default dashboard</description>
    <primary>false</primary>
    <columns>
      <com.pmease.quickbuild.web.page.dashboard.LayoutColumn>
        <width>100</width>
        <gadgetDOMs>
          <com.pmease.quickbuild.web.gadget.ConfigurationTreeGadget revision="0.0.1">
            <title>All Configurations</title>
            <treeRoot>1</treeRoot>
            <displayTreeRoot>true</displayTreeRoot>
            <displaySchedule>false</displaySchedule>
            <displayRequestCount>true</displayRequestCount>
            <displayHistoryCount>true</displayHistoryCount>
          </com.pmease.quickbuild.web.gadget.ConfigurationTreeGadget>
        </gadgetDOMs>
      </com.pmease.quickbuild.web.page.dashboard.LayoutColumn>
    </columns>
  </com.pmease.quickbuild.model.Dashboard>
</list>
"""

DASHBOARD_INFO_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<com.pmease.quickbuild.model.Dashboard>
  <id>1</id>
  <user>1</user>
  <name>Default</name>
  <description>System default dashboard</description>
  <primary>false</primary>
  <columns>
    <com.pmease.quickbuild.web.page.dashboard.LayoutColumn>
      <width>100</width>
      <gadgetDOMs>
        <com.pmease.quickbuild.web.gadget.ConfigurationTreeGadget revision="0.0.1">
          <title>All Configurations</title>
          <treeRoot>1</treeRoot>
          <displayTreeRoot>true</displayTreeRoot>
          <displaySchedule>false</displaySchedule>
          <displayRequestCount>true</displayRequestCount>
          <displayHistoryCount>true</displayHistoryCount>
        </com.pmease.quickbuild.web.gadget.ConfigurationTreeGadget>
        <com.pmease.quickbuild.plugin.report.changes.gadget.CommitStatsGadget revision="0.0.0.0.0.0">
          <title>Commits</title>
          <configurationPath>root</configurationPath>
          <reportsets/>
          <indicators>
            <string>Commits</string>
            <string>Modifications</string>
          </indicators>
          <groupBy>BY_DAY</groupBy>
          <excludingFailed>false</excludingFailed>
          <ignoreNoBuildDays>false</ignoreNoBuildDays>
        </com.pmease.quickbuild.plugin.report.changes.gadget.CommitStatsGadget>
      </gadgetDOMs>
    </com.pmease.quickbuild.web.page.dashboard.LayoutColumn>
  </columns>
</com.pmease.quickbuild.model.Dashboard>
"""

DASHBOARD_INFO_JSON = """{
  "id" : 1,
  "user" : 1,
  "name" : "Default",
  "description" : "System default dashboard",
  "primary" : false,
  "columns" : [ {
    "width" : 100,
    "gadgetDOMs" : [ {
      "title" : "All Configurations",
      "treeRoot" : 1,
      "displayTreeRoot" : true,
      "displaySchedule" : false,
      "displayRequestCount" : true,
      "displayHistoryCount" : true,
      "@class" : "com.pmease.quickbuild.web.gadget.ConfigurationTreeGadget"
    }, {
      "title" : "Commits",
      "configurationPath" : "root",
      "reportsets" : [ ],
      "indicators" : [ "Commits", "Modifications" ],
      "groupBy" : "BY_DAY",
      "excludingFailed" : false,
      "ignoreNoBuildDays" : false,
      "@class" : "com.pmease.quickbuild.plugin.report.changes.gadget.CommitStatsGadget"
    } ]
  } ]
}
"""


@responses.activate
def test_get(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/dashboards'),
        content_type='application/xml',
        body=DASHBOARDS_XML,
    )

    response = client.dashboards.get()
    assert len(response) == 1
    assert response[0]['id'] == 1
    assert response[0]['user'] == 1
    assert response[0]['name'] == 'Default'
    assert response[0]['primary'] is False
    assert response[0]['columns'][0]['width'] == 100


@responses.activate
def test_get_info(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/dashboards'),
        content_type='application/xml',
        body=DASHBOARD_INFO_XML,
    )

    response = client.dashboards.get_info(1)

    assert response['@class'] == 'com.pmease.quickbuild.model.Dashboard'
    assert response['id'] == 1
    assert response['columns'][0]['gadgetDOMs'][0]['revision'] == '0.0.1'


@responses.activate
def test_update(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/dashboards'),
        content_type='application/xml',
        body='10',
    )

    response = client.dashboards.update(DASHBOARD_INFO_XML)
    assert response == 10


@responses.activate
def test_create(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/dashboards'),
        content_type='application/xml',
        body='11',
    )

    configuration_without_id = DASHBOARD_INFO_XML.replace('<id>1</id>', '')
    response = client.dashboards.create(configuration_without_id)
    assert response == 11

    with pytest.raises(QBError):
        client.dashboards.create(DASHBOARD_INFO_XML)

    with pytest.raises(QBError):
        client.dashboards.create(DASHBOARD_INFO_JSON)


@responses.activate
def test_delete(client):
    responses.add(
        responses.DELETE,
        re.compile(r'.*/rest/dashboards'),
        body='12',
    )

    response = client.dashboards.delete(12)
    assert response == 12



from typing import List, Union

from quickbuild.helpers import response2py


class Agents:

    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def get_active(self) -> Union[List[dict], str]:
        """
        Get list of active build agents.

        Returns:
            List[dict]: list of active build agents.
        """
        return self.quickbuild._request(
            'GET',
            'buildagents/active',
            callback=response2py,
        )

    def get_inactive(self) -> Union[List[dict], str]:
        """
        Get list of inactive build agents.

        Returns:
            List[dict]: list of inactive build agents.
        """
        return self.quickbuild._request(
            'GET',
            'buildagents/inactive',
            callback=response2py,
        )

    def get_unauthorized(self) -> Union[List[dict], str]:
        """
        Get list of unauthorized build agents.

        Returns:
            List[dict]: list of unauthorized build agents.
        """
        return self.quickbuild._request(
            'GET',
            'buildagents/unauthorized',
            callback=response2py,
        )

    def get_running_steps(self, node_address: str) -> Union[List[dict], str]:
        """
        Get list of running steps on specified build agent (since 5.1.24).

        Returns:
            List[dict]: list of running steps.
        """
        return self.quickbuild._request(
            'GET',
            'buildagents/{}/running_steps'.format(node_address),
            callback=response2py,
        )



import re

import responses

USER_SHARES_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.model.UserShare>
    <id>1</id>
    <user>2</user>
    <dashboard>1</dashboard>
  </com.pmease.quickbuild.model.UserShare>
  <com.pmease.quickbuild.model.UserShare>
    <id>2</id>
    <user>1</user>
    <dashboard>1</dashboard>
  </com.pmease.quickbuild.model.UserShare>
</list>
"""

USER_SHARE_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<com.pmease.quickbuild.model.UserShare>
  <id>1</id>
  <user>2</user>
  <dashboard>1</dashboard>
</com.pmease.quickbuild.model.UserShare>
"""

GROUP_SHARES_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.model.GroupShare>
    <id>1</id>
    <group>2</group>
    <dashboard>1</dashboard>
  </com.pmease.quickbuild.model.GroupShare>
  <com.pmease.quickbuild.model.GroupShare>
    <id>2</id>
    <group>1</group>
    <dashboard>1</dashboard>
  </com.pmease.quickbuild.model.GroupShare>
</list>
"""

GROUP_SHARE_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<com.pmease.quickbuild.model.GroupShare>
  <id>1</id>
  <group>2</group>
  <dashboard>1</dashboard>
</com.pmease.quickbuild.model.GroupShare>
"""


@responses.activate
def test_get_user_shares(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/user_shares'),
        content_type='application/xml',
        body=USER_SHARES_XML,
    )

    response = client.shares.users.get()
    assert len(response) == 2
    assert response[0]['user'] == 2
    assert client.shares.users.get() == client.users.shares.get()


@responses.activate
def test_get_user_share_by_id(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/user_shares/1'),
        content_type='application/xml',
        body=USER_SHARE_XML,
    )

    response = client.shares.users.get_by_id(1)
    assert response['user'] == 2
    assert client.shares.users.get_by_id(1) == client.users.shares.get_by_id(1)


@responses.activate
def test_get_user_shares_by_dashboard_id(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/user_shares'),
        content_type='application/xml',
        body=USER_SHARES_XML,
    )

    response = client.shares.users.get_by_dashboard_id(1)
    assert response[0]['user'] == 2


@responses.activate
def test_get_user_shares_by_user_id(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/user_shares'),
        content_type='application/xml',
        body=USER_SHARES_XML,
    )

    response = client.shares.users.get_by_user_id(1)
    assert response[0]['user'] == 2


@responses.activate
def test_update_user_share(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/user_shares'),
        body='123',
    )

    response = client.shares.users.update(USER_SHARE_XML)
    assert response == 123


@responses.activate
def test_create_user_share(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/user_shares'),
        body='124',
    )

    response = client.shares.users.create(USER_SHARE_XML)
    assert response == 124


@responses.activate
def test_delete_user_share(client):
    responses.add(
        responses.DELETE,
        re.compile(r'.*/rest/user_shares/.+'),
    )

    client.shares.users.delete(124)


""" ################################ GROUPS ################################ """


@responses.activate
def test_get_group_shares(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/group_shares'),
        content_type='application/xml',
        body=GROUP_SHARES_XML,
    )

    response = client.shares.groups.get()
    assert len(response) == 2
    assert response[0]['group'] == 2
    assert client.shares.groups.get() == client.groups.shares.get()


@responses.activate
def test_get_group_share_by_id(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/group_shares'),
        content_type='application/xml',
        body=GROUP_SHARE_XML,
    )

    response = client.shares.groups.get_by_id(15)
    assert response['id'] == 1
    assert client.shares.groups.get_by_id(15) == client.groups.shares.get_by_id(15)


@responses.activate
def test_get_group_shares_by_dashboard_id(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/group_shares'),
        content_type='application/xml',
        body=GROUP_SHARES_XML,
    )

    response = client.shares.groups.get_by_dashboard_id(1)
    assert len(response) == 2
    assert response[0]['group'] == 2
    assert client.shares.groups.get() == client.groups.shares.get()


@responses.activate
def test_get_group_shares_by_group_id(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/group_shares'),
        content_type='application/xml',
        body=GROUP_SHARES_XML,
    )

    response = client.shares.groups.get_by_group_id(1)
    assert len(response) == 2
    assert response[0]['group'] == 2
    assert client.shares.groups.get() == client.groups.shares.get()


@responses.activate
def test_update_group_share(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/group_shares'),
        body='123',
    )

    response = client.shares.groups.update(GROUP_SHARE_XML)
    assert response == 123


@responses.activate
def test_create_group_share(client):
    responses.add(
        responses.POST,
        re.compile(r'.*/rest/group_shares'),
        body='123',
    )

    response = client.shares.groups.create(GROUP_SHARE_XML)
    assert response == 123


@responses.activate
def test_delete_group_share(client):
    responses.add(
        responses.DELETE,
        re.compile(r'.*/rest/group_shares/.+'),
    )

    client.shares.groups.delete(124)



from quickbuild.adapters.aio import AsyncQBClient
from quickbuild.adapters.sync import QBClient
from quickbuild.exceptions import (
    QBError,
    QBForbiddenError,
    QBNotFoundError,
    QBProcessingError,
    QBServerError,
)
from quickbuild.helpers import ContentType

__version__ = '0.16.0'

__all__ = (
    # adapters
    'AsyncQBClient',
    'QBClient',
    # exceptions
    'QBError',
    'QBForbiddenError',
    'QBNotFoundError',
    'QBProcessingError',
    'QBServerError',
    # other
    'ContentType',
)



from typing import List, Union

from quickbuild.helpers import response2py


class Shares:
    """
    Share is the object used to control dashboard sharing with users or groups.
    """
    def __init__(self, quickbuild):
        self.users = UserShares(quickbuild)
        self.groups = GroupShares(quickbuild)


class UserShares:
    """
    User share is the object used to control dashboard sharing with users.

    https://wiki.pmease.com/display/QB10/Interact+with+User+Share
    """
    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def get(self) -> Union[List[dict], str]:
        """
        Get all user shares in the system.

        Returns:
            Union[List[dict], str]: list of all user shares.
        """
        return self.quickbuild._request(
            'GET',
            'user_shares',
            callback=response2py
        )

    def get_by_id(self, share_id: int) -> Union[dict, str]:
        """
        Get user share by identifier.

        Args:
            share_id (int):
                Share identifier.

        Returns:
            Union[dict, str]: user share.
        """
        return self.quickbuild._request(
            'GET',
            'user_shares/{}'.format(share_id),
            callback=response2py
        )

    def get_by_dashboard_id(self, dashboard_id: int) -> Union[List[dict], str]:
        """
        Get user shares by dashboard identifier.

        Args:
            dashboard_id (int):
                Dashboard identifier.

        Returns:
            Union[List[dict], str]: list of user shares.
        """
        return self.quickbuild._request(
            'GET',
            'user_shares',
            callback=response2py,
            params=dict(dashboard_id=dashboard_id),
        )

    def get_by_user_id(self, user_id: int) -> Union[List[dict], str]:
        """
        Get user shares by user identifier.

        Args:
            user_id (int):
                User identifier.

        Returns:
            Union[List[dict], str]: list of user shares.
        """
        return self.quickbuild._request(
            'GET',
            'user_shares',
            callback=response2py,
            params=dict(user_id=user_id),
        )

    def update(self, configuration: str) -> int:
        """
        Update user share using XML configuration.

        Normally you do not need to create the XML from scratch: you may retrieve
        XML representation of the share, modify certain parts of the XML and post
        back to above url.

        Args:
            configuration (str): XML document.

        Returns:
            int: user share id being updated.
        """
        return self.quickbuild._request(
            'POST',
            'user_shares',
            callback=response2py,
            data=configuration
        )

    def create(self, configuration: str) -> int:
        """
        Create user share using XML configuration.

        Normally you do not need to create the XML from scratch: you may retrieve
        XML representation of the share, modify certain parts of the XML and post
        back to above url.

        Args:
            configuration (str): XML document.

        Returns:
            int: user share id being created.
        """
        return self.update(configuration)

    def delete(self, share_id: str) -> None:
        """
        Delete the user share.

        Args:
            share_id (int):
                Share identifier.

        Returns:
            int: user share id being deleted.
        """
        return self.quickbuild._request(
            'DELETE',
            'user_shares/{}'.format(share_id),
            callback=response2py,
        )


class GroupShares:
    """
    Group share is the object used to control dashboard sharing with groups.
    """
    def __init__(self, quickbuild):
        self.quickbuild = quickbuild

    def get(self) -> Union[List[dict], str]:
        """
        Get all group shares in the system.

        Returns:
            Union[List[dict], str]: list of all group shares.
        """
        return self.quickbuild._request(
            'GET',
            'group_shares',
            callback=response2py
        )

    def get_by_id(self, share_id: int) -> Union[dict, str]:
        """
        Get group share by identifier.

        Args:
            share_id (int):
                Share identifier.

        Returns:
            Union[dict, str]: user share.
        """
        return self.quickbuild._request(
            'GET',
            'group_shares/{}'.format(share_id),
            callback=response2py
        )

    def get_by_dashboard_id(self, dashboard_id: int) -> Union[List[dict], str]:
        """
        Get group shares by dashboard identifier.

        Args:
            dashboard_id (int):
                Dashboard identifier.

        Returns:
            Union[List[dict], str]: list of group shares.
        """
        return self.quickbuild._request(
            'GET',
            'group_shares',
            callback=response2py,
            params=dict(dashboard_id=dashboard_id),
        )

    def get_by_group_id(self, group_id: int) -> Union[List[dict], str]:
        """
        Get group shares by group identifier.

        Args:
            group_id (int):
                Represents id of the group to query group share information for.
                Particularly, use id 0 to query group shares targeting everyone.

        Returns:
            Union[List[dict], str]: list of group shares.
        """
        return self.quickbuild._request(
            'GET',
            'group_shares',
            callback=response2py,
            params=dict(group_id=group_id),
        )

    def update(self, configuration: str) -> int:
        """
        Update group share using XML configuration.

        Normally you do not need to create the XML from scratch: you may retrieve
        XML representation of the share, modify certain parts of the XML and post
        back to above url.

        Args:
            configuration (str): XML document.

        Returns:
            int: group share id being updated.
        """
        return self.quickbuild._request(
            'POST',
            'group_shares',
            callback=response2py,
            data=configuration
        )

    def create(self, configuration: str) -> int:
        """
        Create group share using XML configuration.

        Normally you do not need to create the XML from scratch: you may retrieve
        XML representation of the share, modify certain parts of the XML and post
        back to above url.

        Args:
            configuration (str): XML document.

        Returns:
            int: group share id being updated.
        """
        return self.update(configuration)

    def delete(self, share_id: str) -> None:
        """
        Delete group share.

        Args:
            share_id (int):
                Share identifier.

        Returns:
            int: group share id being deleted.
        """
        return self.quickbuild._request(
            'DELETE',
            'group_shares/{}'.format(share_id),
            callback=response2py,
        )



import re

import responses

ISSUES_LIST_XML = r"""
<report name="issues" version="0.0" locale="en_US">
  <row ID="15" issueKey="TST-23" buildId="321" issueType="New Feature" summary="2-phase load components" status="Resolved" priority="Major" created="2011-06-03T14:58:47.445+08:00" updated="2011-06-03T15:27:39.418+08:00" resolution="Fixed" assignee="admin" reporter="admin" modifications="3">
    <changeIds>
      913fcbcdbe8f80d6c1fe5fce3bb49e1f0040a943,f090cd04725c5551f8a439fe0a53591193ea79c3
    </changeIds>
    <repositories>hg,hg</repositories>
  </row>
  <row ID="16" issueKey="TST-24" buildId="322" issueType="New Feature 2" summary="2-phase load components" status="Resolved" priority="Major" created="2021-06-03T14:58:47.445+08:00" updated="2021-06-03T15:27:39.418+08:00" resolution="Fixed" assignee="admin" reporter="admin" modifications="3">
    <changeIds>
      bb49e1f0040a943913fcbcdbe8f80d6c1fe5fce3,04725c5551f8af090cd439fe0a53591193ea79c3
    </changeIds>
    <repositories>hg,hg</repositories>
  </row>
</report>
"""

BUILDS_LIST_XML = r"""
<list>
  <build>
    <id>321</id>
    <version>1.0.15</version>
    <status>SUCCESSFUL</status>
    <beginDate>2011-06-10T17:11:35.662+08:00</beginDate>
    <duration>3409</duration>
    <scheduled>false</scheduled>
    <requester>Administrator</requester>
    <deleted>false</deleted>
  </build>
</list>
"""

CHANGES_LIST_XML = r"""
<list>
  <changeset>
    <user>steve</user>
    <date>2011-05-31T03:45:55.000+08:00</date>
    <id>f090cd04725c5551f8a439fe0a53591193ea79c3</id>
    <repositoryName>hg</repositoryName>
    <repositoryType>Mercurial</repositoryType>
    <additional>13</additional>
    <buildId>321</buildId>
    <comment>
       add a big file to related to issue TST-23, TST-24 TST-25
    </comment>
    <modifications>
      <modification>
        <action>ADD</action>
        <path>big.java</path>
        <edition>f090cd04725c5551f8a439fe0a53591193ea79c3</edition>
        <previousEdition>c11a08aa7525a01e35239b8b6f34d4f8f3bf770b</previousEdition>
        <additional>13</additional>
      </modification>
    </modifications>
  </changeset>
</list>
"""

@responses.activate
def test_get_version(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/jira/version'),
        content_type='text/plain',
        body='3.1',
    )

    version = client.issues.get_tracker('jira').get_version()
    assert version == '3.1'


@responses.activate
def test_get_size(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/jira/size/.*'),
        content_type='text/plain',
        body='29',
    )

    size = client.issues.get_tracker('jira').get_size(
        'root/My/DEV',
        from_build=100,
        to_build=150,
    )

    assert size == 29


@responses.activate
def test_get_issues(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/jira/issues/.*'),
        content_type='text/xml',
        body=ISSUES_LIST_XML,
    )

    issues = client.issues.get_tracker('jira').get_issues(
        19,
        build=100,
    )

    assert len(issues) == 2
    assert issues[0]['assignee'] == 'admin'


@responses.activate
def test_get_builds(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/jira/.+/builds/.+'),
        content_type='text/xml',
        body=BUILDS_LIST_XML,
    )

    builds = client.issues.get_tracker('jira').get_builds(5, 'QB-123')

    assert len(builds) == 1
    assert builds[0]['id'] == 321


@responses.activate
def test_get_changes(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/jira/.+/changes/.+'),
        content_type='text/xml',
        body=CHANGES_LIST_XML,
    )

    builds = client.issues.get_tracker('jira').get_changes(5, 'QB-123')

    assert len(builds) == 1
    assert builds[0]['user'] == 'steve'



import re

import responses

CHANGESETS_XML = r"""<list>
  <changeset>
    <user>steve</user>
    <date>2011-06-03T15:11:40.000+08:00</date>
    <id>913fcbcdbe8f80d6c1fe5fce3bb49e1f0040a943</id>
    <comment>
      Fix TST-23: 2-phase load components
    </comment>
    <repositoryName>hg</repositoryName>
    <repositoryType>Mercurial</repositoryType>
    <additional>14</additional>
    <buildId>321</buildId>

    <modifications>
      <modification>
        <action>MODIFY</action>
        <path>Test.java</path>
        <edition>913fcbcdbe8f80d6c1fe5fce3bb49e1f0040a943</edition>
        <previousEdition>f090cd04725c5551f8a439fe0a53591193ea79c3</previousEdition>
        <additional>14</additional>
      </modification>
      <modification>
        <action>MODIFY</action>
        <path>big.java</path>
        <edition>913fcbcdbe8f80d6c1fe5fce3bb49e1f0040a943</edition>
        <previousEdition>f090cd04725c5551f8a439fe0a53591193ea79c3</previousEdition>
        <additional>14</additional>
      </modification>
    </modifications>
  </changeset>
</list>
"""


@responses.activate
def test_get_version(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/changes/version'),
        content_type='text/plain',
        body='3.2',
    )

    version = client.changes.get_version()
    assert version == '3.2'


@responses.activate
def test_get_commit_stats(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/changes/stats/.+'),
        content_type='text/xml',
        body='<stats commits="5" modifications="7" added="2" modified="3" deleted="2"/>',
    )

    stats = client.changes.get_commit_stats(
        1,
        build=9,
        from_build=1,
        to_build=333,
        from_date='19911209',
        to_date='20211209',
        date_pattern='yyyyMMdd',
        repository='main',
        committer='p.belskiy',
    )

    assert stats['commits'] == 5
    assert stats['modifications'] == 7
    assert stats['added'] == 2
    assert stats['modified'] == 3
    assert stats['deleted'] == 2


@responses.activate
def test_get_changesets(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/changes/.+'),
        content_type='text/xml',
        body=CHANGESETS_XML,
    )

    changesets = client.changes.get_changesets(
        1,
        build=9,
        from_build=1,
        to_build=333,
        from_date='19911209',
        to_date='20211209',
        date_pattern='yyyyMMdd',
        repository='main',
        committer='p.belskiy',
        offset=1,
        limit=100,
        asc=False,
    )

    assert changesets[0]['user'] == 'steve'


@responses.activate
def test_get_build_commits_changesets(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/changes/commits/build/.+'),
        content_type='text/xml',
        body=CHANGESETS_XML,
    )

    changesets = client.changes.get_build_commits_changesets(123)
    assert changesets[0]['user'] == 'steve'


@responses.activate
def test_get_build_stats_changesets(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/changes/stats/build/.+'),
        content_type='text/xml',
        body=CHANGESETS_XML,
    )

    changesets = client.changes.get_build_stats_changesets(123)
    assert changesets[0]['user'] == 'steve'



import re

import responses

UNAUTHORIZED_AGENTS_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.grid.UnauthorizedAgent>
    <ip>172.17.0.1</ip>
    <port>8811</port>
    <overSSL>false</overSSL>
    <hostName>live.local</hostName>
    <lastPulse>2021-07-02T21:20:19.393Z</lastPulse>
  </com.pmease.quickbuild.grid.UnauthorizedAgent>
</list>
"""

AGENTS_XML = r"""<?xml version="1.0" encoding="UTF-8"?>

<list>
  <com.pmease.quickbuild.model.Token>
    <id>1</id>
    <ip>172.17.0.1</ip>
    <port>8811</port>
    <test>false</test>
    <hostName>live.local</hostName>
    <offlineAlert>true</offlineAlert>
  </com.pmease.quickbuild.model.Token>
</list>
"""


@responses.activate
def test_get_active(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/buildagents/active'),
        content_type='application/xml',
        body=AGENTS_XML,
    )

    response = client.agents.get_active()
    assert response[0]['id'] == 1
    assert response[0]['ip'] == '172.17.0.1'


@responses.activate
def test_get_inactive(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/buildagents/inactive'),
        content_type='application/xml',
        body=AGENTS_XML,
    )

    response = client.agents.get_inactive()
    assert response[0]['id'] == 1
    assert response[0]['ip'] == '172.17.0.1'


@responses.activate
def test_get_unauthorized(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/buildagents/unauthorized'),
        content_type='application/xml',
        body=UNAUTHORIZED_AGENTS_XML,
    )

    response = client.agents.get_unauthorized()
    assert response[0]['ip'] == '172.17.0.1'
    assert response[0]['overSSL'] is False


@responses.activate
def test_get_running_steps(client):
    responses.add(
        responses.GET,
        re.compile(r'.*/rest/buildagents/.+/running_steps'),
        content_type='application/xml',
    )

    client.agents.get_running_steps('myagent:8811')
